from rest_framework.views import APIView
from ..services.payment_unionpay_service import PaymentUnionPayService


class PaymentUnionPay(APIView):
    def unipay(self):
        res = PaymentUnionPayService.unipay("200", "1")
        return res



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 返回协议封装工具
@created_time: 2022/6/15 14:14
"""
import datetime
import decimal
import json
import re
import traceback
import uuid

from django.http import JsonResponse
from django.utils.duration import duration_iso_string
from django.utils.functional import Promise
from django.utils.timezone import is_aware

from .custom_tool import write_to_log


class CustomJSONEncoder(json.JSONEncoder):
    def default(self, o):
        # See "Date Time String Format" in the ECMA-262 specification.
        if isinstance(o, datetime.datetime):
            r = o.isoformat()
            if o.microsecond:
                r = r[:23] + r[26:]
            if r.endswith('+00:00'):
                r = r[:-6] + 'Z'
            return r
        elif isinstance(o, datetime.date):
            return o.isoformat()
        elif isinstance(o, datetime.time):
            if is_aware(o):
                raise ValueError("JSON can't represent timezone-aware times.")
            r = o.isoformat()
            if o.microsecond:
                r = r[:12]
            return r
        elif isinstance(o, datetime.timedelta):
            return duration_iso_string(o)
        elif isinstance(o, (decimal.Decimal)):
            return float(o)

        elif isinstance(o, (decimal.Decimal, uuid.UUID, Promise)):
            return str(o)
        else:
            return super().default(o)


# json 结果集返回
def parse_json_str(result):
    if not result is None:
        if type(result) is str:
            # Replace common line and tab characters with spaces
            result = result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', "")
            try:
                # Load the string as JSON and convert it back to a string
                result = json.loads(result)
                result = json.dumps(result)
            except Exception as e:
                return result
        elif type(result) is list or type(result) is tuple:
            for index, value in enumerate(result):
                result[index] = parse_json_str(value)
        elif type(result) is dict:
            for k, v in result.items():
                result[k] = parse_json_str(v)
    return result

# 数据返回规则
def util_response(data='', err=0, msg='ok', is_need_parse_json=True, cookies=None):
    """
    http 返回协议封装
    :param is_need_parse_json: 是否需要解析json字符串
    :param cookies: 设置cookie的json
    :param data: 返回的数据体
    :param err: 错误码，一般以1000开始，逐一增加。登录错误为6000-6999。
    :param msg: 错误信息，一般为服务返回协议中的err,自动解析内容
    :return: response对象
    """
    # ============= section 构建body的json数据 start =================
    print(">>>",data)
    data = parse_json_str(data) if is_need_parse_json else data
    response_json = {'err': err, 'data': data}
    # 解析msg字符串
    try:
        msg_list = msg.split(";")
        if len(msg_list) <= 1:
            response_json["msg"] = msg
        else:
            for i in msg_list:
                [key, value] = i.split(":")
                response_json[key] = value
    except Exception as e:
        # 获取谁调用该函数
        try:
            call_file_obj = traceback.extract_stack()[-2]
            call_file_name = str(call_file_obj.filename)
            call_file_line = str(call_file_obj.lineno)
        except Exception as get_call_file_err:
            call_file_name = ""
            call_file_line = ""
        # 写入日志
        write_to_log(
            prefix="返回消息异常，请提前处理好份号‘：’，请及时修改",
            content="msg:" + str(msg) + "调用文件：" + call_file_name + "  行数：" + call_file_line
        )
        response_json["msg"] = msg
    # ============= section 构建body的json数据 end =================

    # ============= section 构建响应对象 start =================
    response = JsonResponse(response_json, encoder=CustomJSONEncoder)
    # 设置cookie
    if cookies and isinstance(cookies, dict):
        for k, v in cookies.items():
            response.set_cookie(k, value=v)
    # ============= section 构建响应对象 end =================
    return response



import logging
import random
import sys
from datetime import datetime
import decimal
import json
import pytz
import requests
import time
from decimal import Decimal
from dateutil.parser import parse
from dateutil.tz import tzlocal
from django.core.paginator import Paginator
from django.db.models import F
from django.db import transaction
from django.forms import model_to_dict
from django.utils import timezone
from pathlib import Path
from main.settings import BASE_DIR
from xj_enroll.service.enroll_services import EnrollServices
from xj_finance.services.finance_transact_service import FinanceTransactService
from xj_payment.utils.join_list import JoinList
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.models import Platform
from xj_user.services.user_platform_service import UserPlatformService
from xj_user.services.user_sso_serve_service import UserSsoServeService
from .payment_alipay_service import PaymentAlipayService
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.custom_tool import format_list_handle, format_params_handle, write_to_log
from ..models import PaymentPayment, PaymentStatus
from ..services.payment_wechat_service import PaymentWechatService
from ..utils.utility_method import find, format_dates

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

# 小程序appid
sub_appid = main_config_dict.wechat_merchant_app_id or module_config_dict.wechat_merchant_app_id or ""
# 服务号appid
subscription_app_id = main_config_dict.wechat_subscription_app_id or module_config_dict.wechat_subscription_app_id or ""
# 商户名称
merchant_name = main_config_dict.merchant_name or module_config_dict.merchant_name or ""


class DateEncoder(json.JSONEncoder):
    def default(self, obj):
        try:
            if isinstance(obj, int):
                return int(obj)
            elif isinstance(obj, float) or isinstance(obj, decimal.Decimal):
                return float(obj)
            if isinstance(obj, datetime.datetime):
                return obj.strftime("%Y-%m-%d %H:%M:%S")
            elif isinstance(obj, datetime.date):
                return obj.strftime('%Y-%m-%d')
            if isinstance(obj, time) or isinstance(obj, datetime.timedelta):
                return obj.__str__()
            else:
                return json.JSONEncoder.default(self, obj)
        except Exception as e:
            # logger.exception(e, stack_info=True)
            return obj.__str__()


class PaymentLogicalProcessService:
    # 支付逻辑处理
    @staticmethod
    @transaction.atomic
    def payment_logic_processing(param, pay_mode="WECHAT"):
        sid = transaction.savepoint()
        try:
            out_trade_no = param['out_trade_no']  # 订单号
            total_fee = param['total_fee']  # 金额（单位分）
            total_amount = int(total_fee) / 100  # 分转元
            # if pay_mode != "WECHAT":
            #     total_amount = total_fee
            transact_no = 0
            if 'transaction_id' in param:
                transact_no = param['transaction_id']  # 支付订单号
            data = {
                "order_no": out_trade_no,
                "amount": total_amount,
                "pay_mode": pay_mode
            }
            # 根据订单号查询支付记录是否存在
            payment = PaymentPayment.objects.filter(order_no=int(out_trade_no)).first()
            if not payment:
                write_to_log(
                    prefix="out_trade_no" + str(out_trade_no) + "支付记录不存在",
                )
            payment_message = model_to_dict(payment)
            data['account_id'] = payment_message['user_id']
            data['enroll_id'] = payment_message['enroll_id']
            goods_info = {}
            data['goods_info'] = goods_info
            # # TODO 拿到订单号后的操作 看自己的业务需求
            # 写入资金模块
            finance, err_txt = FinanceTransactService.finance_flow_writing(params=data, finance_type='TOP_UP')
            if err_txt:
                write_to_log(
                    prefix=str(data) + "写入资金模块失败",
                    content=err_txt
                )
            # 根据唯一交易id 查询主键id
            payment_status_set = PaymentStatus.objects.filter(payment_status="成功").first()
            payment_data = {
                "transact_no": transact_no,
                "payment_time": param.get("payment_time", None) if param.get("payment_time", None) else None,
                "payment_status": payment_status_set.id,
                "pay_mode": pay_mode,
                "order_status_id": 2,

            }
            # 更改支付记录
            PaymentPayment.objects.filter(order_no=int(out_trade_no)).update(**payment_data)
            # 报名表支付状态修改
            pay_call_back_data, pay_call_back_err = EnrollServices.bxtx_pay_call_back(out_trade_no)
            if pay_call_back_err:
                write_to_log(
                    prefix="报名表支付状态修改",
                    content="out_trade_no:" + str(out_trade_no),
                    err_obj=pay_call_back_err
                )

        except Exception as err:
            write_to_log(
                prefix="支付回调异常",
                content="",
                err_obj=err
            )
            transaction.savepoint_rollback(sid)



from django.contrib import admin

# 引入用户平台
from .models import *

class PaymentStatusAdmin(admin.ModelAdmin):

    fields = ('id', 'payment_status', 'group', 'description')
    list_display = ('id', 'payment_status', 'group', 'description')
    readonly_fields = ['id']

admin.site.register(PaymentStatus, PaymentStatusAdmin)


class PaymentPaymentAdmin(admin.ModelAdmin):

    fields = ('id', 'enroll_id', 'order_id', 'order_no', 'transact_no', 'transact_id', 'user_id', 'subject', 'total_amount', 'buyer_pay_amount', 'point_amount', 'invoice_amount', 'price_off_amount', 'pay_mode',)
    list_display = ('id','enroll_id', 'order_id', 'order_no', 'transact_no', 'transact_id', 'user_id', 'subject', 'total_amount', 'buyer_pay_amount', 'point_amount', 'invoice_amount', 'price_off_amount', 'pay_mode',)
    readonly_fields = ['id']
    search_fields = ['order_no',"transact_no"]
    list_filter = ['pay_mode']

admin.site.register(PaymentPayment, PaymentPaymentAdmin)



"""
Created on 2023-07-20
@author:高栋天
@description:自定义方法
"""
import pinyin
import uuid
import json
import sys
import time
import os
import string
import random
import pytz
import datetime
from django.utils import timezone
from dateutil.parser import parse
from dateutil.tz import tzlocal
import pandas as pd
from datetime import datetime
from decimal import Decimal


# 获取当前时间
def get_current_time():
    # TODO USE_TZ = False 时会报错 如果USE_TZ设置为True时，Django会使用系统默认设置的时区，即America/Chicago，此时的TIME_ZONE不管有没有设置都不起作用。
    tz = pytz.timezone('Asia/Shanghai')
    # 返回datetime格式的时间
    now_time = timezone.now().astimezone(tz=tz).strftime("%Y-%m-%d %H:%M:%S.%f")
    # now = datetime.strptime(now_time, '%Y-%m-%d %H:%M:%S')
    now = datetime.now().strptime(now_time, "%Y-%m-%d %H:%M:%S.%f")
    return now


def get_current_time_second():
    # 设置时区为Asia/Shanghai
    tz = pytz.timezone('Asia/Shanghai')

    # 获取当前的UTC时间
    utc_now = datetime.now(pytz.utc)

    # 将UTC时间转换为您指定的时区（Asia/Shanghai）并转换为字符串格式
    now_time = utc_now.astimezone(tz).strftime("%Y-%m-%d %H:%M:%S")

    # 返回当前时间的字符串格式
    return now_time


# 时间精确到微秒
def append_microsecond_to_datetime(datetime_str: str) -> datetime:
    """
    将当前微秒追加到日期时间字符串中。如果字符串仅包含日期，当前时间也附加在微秒之前。
    """
    # Check if the datetime string contains a time
    if len(datetime_str.split(' ')) == 2:
        current_microsecond = datetime.now().strftime("%f")
        datetime_str = f'{datetime_str}.{current_microsecond}'
        dt = datetime.strptime(datetime_str, "%Y-%m-%d %H:%M:%S.%f")
    else:
        # Get the current time and microsecond
        current_time_microsecond = datetime.now().strftime("%H:%M:%S.%f")
        datetime_str = f'{datetime_str} {current_time_microsecond}'
        dt = datetime.strptime(datetime_str, "%Y-%m-%d %H:%M:%S.%f")
    return dt


# 数据key替换
def replace_dict_key(dictionary, old_key, new_key):
    """
       :param dictionary: 字典
       :param old_key: 旧key
       :param new_key: 替换的新key
   """
    if old_key in dictionary:
        dictionary[new_key] = dictionary.pop(old_key)
    return dictionary


# 数据字典key替换
def replace_key_in_dict(original_dict, old_key, new_key):
    """
       :param original_dict: 字典
       :param old_key: 旧key
       :param new_key: 替换的新key
   """
    copied_dict = original_dict.copy()
    if old_key in copied_dict:
        copied_dict[new_key] = copied_dict.pop(old_key)
    return copied_dict


# 数据列表key替换
def replace_key_in_list_dicts(list_dicts, old_keys, new_key):
    """
       :param list_dicts: 列表
       :param old_key: 旧key
       :param new_key: 替换的新key
    """
    for d in list_dicts:
        for old_key in old_keys:
            if old_key in d:
                d[new_key] = d.pop(old_key)
    return list_dicts


# 字典替换字典
def replace_key_in_dict_replacement_dicts(dict_data, replacement_dict):
    """
      :param dict_data: 字典
      :param replacement_dict: 替换的字典映射
   """
    for old_key, new_key in replacement_dict.items():
        if old_key in dict_data:
            dict_data[new_key] = dict_data.pop(old_key, dict_data[old_key])
    return dict_data


# 数据列表字典key替换
def replace_key_in_list_replacement_dicts(list_dicts, replacement_dict):
    """
       :param list_dicts: 列表
       :param replacement_dict: 要替换的字典
    """
    for d in list_dicts:
        for old_key, new_key in replacement_dict.items():
            if old_key in d:
                d[new_key] = d.pop(old_key)
    return list_dicts


# 字符串转列表
# 1. 如果传入的是字符串，它会检查是否有逗号，然后将其分割成列表或者直接转为包含一个元素的列表。
# 2. 如果传入的是整数，它会将整数转换成字符串，并放入一个列表中。
# 3. 如果传入的是列表，它会将列表中的每个元素转换成字符串。
def parse_integers(value):
    if isinstance(value, str):
        if "," in value:
            lst = value.split(",")
        else:
            lst = [value]
    elif isinstance(value, int):
        lst = [str(value)]
    elif isinstance(value, list):
        lst = [str(num) for num in value]
    else:
        raise ValueError("不支持的值类型。应为字符串、int或list")

    return lst


# 保留两位小数
def keep_two_decimal_places(str_num):
    """
    :param str_num: 要处理的字段
    """
    result_num = format(float(str_num), "")

    if len(result_num.split(".")[-1]) < 2:
        result_num = result_num + "0"
    return result_num


# 生成一个长度为16的密码
def generate_password(length=16):
    """
     :param length: 长度
     """
    # 合并所有可能的字符，包括大小写字母、数字和标点符号
    # all_chars = string.ascii_letters + string.digits + string.punctuation
    all_chars = string.ascii_letters + string.digits
    # length = random.randint(8, 12)
    # 随机选择指定数量的字符
    password = ''.join(random.choice(all_chars) for _ in range(length))

    return password


# 数字表示生成几位, True表示生成带有字母的 False不带字母的
def get_code(n=6, alpha=False):
    """
     :param n: 要生成的位数
     :param alpha: True表示生成带有字母的 False不带字母的
     """
    s = ''  # 创建字符串变量,存储生成的验证码
    for i in range(n):  # 通过for循环控制验证码位数
        num = random.randint(1, 9)  # 生成随机数字0-9
        if alpha:  # 需要字母验证码,不用传参,如果不需要字母的,关键字alpha=False
            upper_alpha = chr(random.randint(65, 90))
            lower_alpha = chr(random.randint(97, 122))
            num = random.choice([num, upper_alpha, lower_alpha])
        s = s + str(num)
    return s


# 检查列表字段是否存在
def find(list, keyword):
    """
   :param list: 列表
   :param keyword: 要检查的字段
   """
    try:
        list.index(keyword)
        return True
    except ValueError:
        return False


# 批量数据时间格式化
def format_dates(data, date_fields):
    """
   :param data: 列表
   :param date_fields: 要格式化的字段列表
   """
    for item in data:
        for field in date_fields:
            if field in item and item[field]:
                try:
                    # 如果字段已经是 datetime 对象，就无需解析
                    if isinstance(item[field], datetime):
                        date = item[field]
                    else:
                        # 尝试解析并格式化日期
                        date = parse(item[field])
                    # 使用 strftime 格式化日期
                    item[field] = date.astimezone(tzlocal()).strftime('%Y-%m-%d %H:%M:%S')
                except ValueError:
                    # 如果解析失败，保留原来的值
                    pass
    return data


# 列表根据指定key计算value求和
# def aggregate_data(data_list, group_field, agg_fields):
#     """
#    :param data_list: 列表
#    :param group_field: 分组规则
#    :param agg_fields: 要求和的字段
#    """
#     # 将数据转换为 DataFrame
#     df = pd.DataFrame(data_list)
#
#     # 将累加字段转换为数值型
#     for field in agg_fields:
#         df[field] = pd.to_numeric(df[field])
#
#     # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
#     grouped = df.groupby(group_field)[agg_fields].sum().reset_index()
#
#     # 将结果转换回字典列表
#     grouped_data = grouped.to_dict('records')
#
#     return grouped_data

# def aggregate_data(data_list, group_field, agg_fields):
#     """
#     :param data_list: 列表
#     :param group_field: 分组规则
#     :param agg_fields: 要求和的字段列表
#     """
#     # 将数据转换为 DataFrame
#     df = pd.DataFrame(data_list)
#
#     # 将累加字段转换为数值型
#     for field in agg_fields:
#         df[field] = pd.to_numeric(df[field], errors='coerce')  # 处理可能的NaN值
#
#     # 将NaN值替换为0
#     df.fillna(0, inplace=True)
#
#     # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
#     grouped = df.groupby(group_field)[agg_fields].sum().reset_index()
#
#     # 将结果转换回字典列表
#     grouped_data = grouped.to_dict('records')
#
#     return grouped_data

# def aggregate_data(data_list, group_field, agg_fields):
#     """
#     :param data_list: 列表
#     :param group_field: 分组规则
#     :param agg_fields: 要求和的字段列表
#     """
#     # 将数据转换为 DataFrame
#     df = pd.DataFrame(data_list)
#
#     # 将累加字段转换为数值型
#     for field in agg_fields:
#         df[field] = pd.to_numeric(df[field], errors='coerce')  # 处理可能的NaN值
#
#     # 将NaN值替换为0
#     df.fillna(0, inplace=True)
#
#     # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
#     grouped = df.groupby(group_field, as_index=False)[agg_fields].sum()
#
#     return grouped.to_dict('records')

def aggregate_data(data_list, group_field, agg_fields, agg_result_field):
    """
    :param data_list: 列表
    :param group_field: 分组规则
    :param agg_fields: 要求和的字段列表
    """
    # 将数据转换为 DataFrame
    df = pd.DataFrame(data_list)

    # 将累加字段转换为数值型
    for field in agg_fields:
        df[field] = pd.to_numeric(df[field], errors='coerce')  # 处理可能的NaN值

    # 将NaN值替换为0
    df.fillna(0, inplace=True)

    # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
    df[agg_result_field] = df.groupby(group_field)[agg_fields].transform('sum')

    # 选择需要的字段
    grouped_data = df.drop(agg_fields, axis=1).drop_duplicates(subset=group_field).to_dict('records')

    return grouped_data


# json模板替换
def replace_placeholders(data, replacements):
    """
   :param data: 要替换的数据
   :param replacements: 要替换的字典
   """
    # 将 data 转换为 JSON 格式的字符串
    data_str = str(data)

    # 依次替换每一个 "{{}}"
    for replacement in replacements:
        data_str = data_str.replace("{{}}", replacement, 1)

    # 将字符串重新转换为字典
    data = eval(data_str)

    return data


# 获取程序运行时间
def testRunTime():
    start = datetime.now()
    for i in range(1000):
        for j in range(500):
            m = i + j
            print(m)
    end = datetime.now()
    print(end - start)
    return end - start


# 将dict转换为json
def convert_dict_to_json(data):
    """
     :param data: 要转换的字典
     """

    def convert_values_to_str(value):
        if isinstance(value, dict):
            return {convert_values_to_str(key): convert_values_to_str(val) for key, val in value.items()}
        elif isinstance(value, list):
            return [convert_values_to_str(val) for val in value]
        elif isinstance(value, Decimal):
            return str(value)
        elif value is None:
            return value
        else:
            return str(value)

    converted_data = convert_values_to_str(data)
    json_str = json.dumps(converted_data, ensure_ascii=False)
    return json_str


# for 列表搜索
def search_in_dict_list(dict_list, query_dict):
    """
     :param dict_list: 要过滤的列表
     :param query_dict: 要筛选的字典
     """
    return [d for d in dict_list if all(d.get(k) == v for k, v in query_dict.items())]


# Pandas库 列表搜索(支持模糊搜索)
def search_in_dict_list(dict_list, query_dict):
    """
   :param dict_list: 要过滤的列表
   :param query_dict: 要筛选的字典
   """
    df = pd.DataFrame(dict_list)
    original_length = len(df)

    for key, value in query_dict.items():
        if isinstance(value, str):
            df = df[df[key].str.contains(value)]
        else:
            df = df[df[key] == value]

    not_matching_count = original_length - len(df)
    return df.to_dict('records'), not_matching_count


# 字典过滤
def filter_dict_by_keys(dict_obj, keys):
    """
    :param dict_obj: 要过滤的字典
    :param keys: 要筛选的字段列表
    """
    keys_set = set(keys)  # Convert list to set
    return {k: v for k, v in dict_obj.items() if k in keys_set}


# 列表字段提取
def extract_values(dict_list, key):
    """
    :param dict_list: 要提取的列表
    :param key: 需要提前的key
    """
    if not dict_list:  # 如果dict_list为空，则返回空列表
        return []
    df = pd.DataFrame(dict_list)
    return df[key].tolist()


# 替换sql里面的占位符
def generate_query_string(query_string, param_list):
    """
      :param query_string: 要替换的字符串
      :param param_list: 替换的列表
      """
    # 替换问号
    for param in param_list:
        query_string = query_string.replace('?', str(param), 1)
    return query_string


# 根据支付宝微信规则生成28位不重复的交易号
def generate_trade_no():
    # 生成不重复的UUID
    trade_no = str(uuid.uuid4().int)
    # 判断交易号长度，不足补0
    if len(trade_no) < 28:
        trade_no = trade_no.zfill(28)
    # 根据支付宝和微信的规则进行格式化
    # trade_no = trade_no[:4] + '-' + trade_no[4:8] + '-' + trade_no[8:12] + '-' + trade_no[12:20] + '-' + trade_no[20:]
    trade_no = trade_no[:4] + trade_no[4:8] + trade_no[8:12] + trade_no[12:20] + trade_no[20:]

    if len(trade_no) < 28:
        trade_no = trade_no.zfill(28)
    return trade_no


# 生成合同编码
def generate_code(code_prefix):
    """
    :param code_prefix: 自定义字符串
    """
    code_date = time.strftime("%Y%m%d%H%M%S", time.localtime())
    code_random = str(random.randint(10000, 99999))
    return f"{code_prefix}_{code_date}_{code_random}"


# 根据中文生成大写缩写
def get_chinese_initials(chinese):
    """
      :param chinese: 中文
    """
    chinese_pinyin = pinyin.get(chinese, format='strip', delimiter=' ')
    initials = [p[0].upper() for p in chinese_pinyin.split()]
    return ''.join(initials)

def convert_data_types(data_list, fields, data_type):
    # 创建一个DataFrame以便进行数据处理
    df = pd.DataFrame(data_list)

    # 遍历要转换的字段列表
    for field in fields:
        # 使用Pandas的astype方法进行类型转换
        df[field] = df[field].astype(data_type)

    # 将DataFrame转换回列表
    result_list = df.to_dict(orient='records')

    return result_list


import logging
import random
import sys
from datetime import datetime

import pytz
from orator import DatabaseManager
import requests
import time
from decimal import Decimal
from dateutil.parser import parse
from dateutil.tz import tzlocal
from django.core.paginator import Paginator
from django.db.models import F
from django.forms import model_to_dict
from django.utils import timezone
from pathlib import Path
from main.settings import BASE_DIR
from xj_enroll.service.enroll_services import EnrollServices
from xj_payment.utils.join_list import JoinList
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.models import Platform
from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.services.user_platform_service import UserPlatformService
from xj_user.services.user_sso_serve_service import UserSsoServeService
from .payment_alipay_service import PaymentAlipayService
from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.custom_tool import format_list_handle, format_params_handle, write_to_log
from ..models import PaymentPayment, PaymentStatus
from ..services.payment_wechat_service import PaymentWechatService
from ..utils.utility_method import find, format_dates, convert_data_types
from config.config import JConfig as JConfigs

module_root = str(Path(__file__).resolve().parent)
# 配置之对象
main_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))
module_config_dict = JDict(JConfig.get_section(path=str(BASE_DIR) + "/config.ini", section="xj_payment"))

# 小程序appid
sub_appid = main_config_dict.wechat_merchant_app_id or module_config_dict.wechat_merchant_app_id or ""
# 服务号appid
subscription_app_id = main_config_dict.wechat_subscription_app_id or module_config_dict.wechat_subscription_app_id or ""
# 商户名称
merchant_name = main_config_dict.merchant_name or module_config_dict.merchant_name or ""

config = JConfigs()
db_config = {
    config.get('main', 'driver', "mysql"): {
        'driver': config.get('main', 'driver', "mysql"),
        'host': config.get('main', 'mysql_host', "127.0.0.1"),
        'database': config.get('main', 'mysql_database', ""),
        'user': config.get('main', 'mysql_user', "root"),
        'password': config.get('main', 'mysql_password', "123456"),
        "port": config.getint('main', 'mysql_port', "3306")
    }
}
db = DatabaseManager(db_config)


class InvalidPage(Exception):
    pass


class PageNotAnInteger(InvalidPage):
    pass


class EmptyPage(InvalidPage):
    pass


class PaymentService:
    @staticmethod
    def golden_touch(params):
        if not params.get("order_no", None):
            return None, "订单号不能为空"
        payemnt = PaymentPayment.objects.filter(**params).order_by('-id').first()
        if payemnt:
            payemnt = model_to_dict(payemnt)
            payemnt['total_amount'] = payemnt['total_amount'] / 100
        return payemnt, None

    @staticmethod
    def get(params):
        limit = params.pop('limit', 20)
        page = params.pop('page', 20)
        list_obj = PaymentPayment.objects.filter(**params).order_by('-id')
        if params.get("create_time_start", None) and params.get("create_time_end", None):
            list_obj = list_obj.filter(
                create_time__range=(params['create_time_start'], params['create_time_end']))

        list_obj = list_obj.annotate(payment_status=F("payment_status__payment_status"),
                                     payment_code=F("payment_status__payment_code"))

        if params.get("payment_status", None):
            list_obj = list_obj.filter(payment_status__payment_status=params.get("payment_status", None), )

        list_obj = list_obj.extra(select={
            'user_full_name': 'SELECT full_name FROM user_base_info WHERE user_base_info.id = payment_payment.user_id'}
        )
        list_obj = list_obj.extra(select={
            'title': 'SELECT thread.title FROM enroll_enroll left join  thread on enroll_enroll.thread_id=thread.id WHERE enroll_enroll.id = payment_payment.enroll_id'}
        )
        count = list_obj.count()
        list_obj = list_obj.values(
            "id",
            "transact_no",
            "order_no",
            "transact_id",
            "enroll_id",
            "order_id",
            "user_id",
            "subject",
            "total_amount",
            "buyer_pay_amount",
            "point_amount",
            "invoice_amount",
            "price_off_amount",
            "pay_mode",
            "order_status_id",
            "payment_status_id",
            "nonce_str",
            "order_time",
            "create_time",
            "modify_time",
            "payment_time",
            "refunt_time",
            "close_time",
            "voucher_detail",
            "snapshot",
            "more",
            "user_full_name",
            "title",
            "payment_status"

        ).annotate(payment_status=F('payment_status__payment_status'), )
        res_set = Paginator(list_obj, limit).get_page(page)
        page_list = []
        if res_set:
            page_list = list(res_set.object_list)
        for v in page_list:
            v['total_amount'] = float(v['total_amount']) / 100 if v['total_amount'] is not None else 0
            v['point_amount'] = float(v['point_amount']) / 100 if v['point_amount'] is not None else 0
            v['buyer_pay_amount'] = float(v['buyer_pay_amount']) / 100 if v['buyer_pay_amount'] is not None else 0
            v['invoice_amount'] = float(v['invoice_amount']) / 100 if v['invoice_amount'] is not None else 0
            v['price_off_amount'] = float(v['price_off_amount']) / 100 if v['price_off_amount'] is not None else 0

        return {'count': count, 'page': page, 'limit': limit, "list": page_list}, None

    @staticmethod
    def list(params, filter_fields=None, need_pagination=True):
        size = params.pop('size', 10)
        page = params.pop('page', 1)
        if params.get("is_all", None):
            params.pop("is_all")
            params.pop("user_id")
        payment_obj = PaymentPayment.objects
        filter_fields_list = filter_fields.split(";") if filter_fields else []
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id", "transact_no", "order_no", "transact_id", "enroll_id", "enroll_id_list", "order_id",
                "user_id", "subject", "total_amount", "buyer_pay_amount", "point_amount",
                "invoice_amount", "price_off_amount", "pay_mode", "order_status_id", "create_time",
                "payment_status_id", "nonce_str", "order_time", "modify_time", "payment_time", "refunt_time",
                "close_time", "payment_status_id",
                "create_time_start", "create_time_end",
                "modify_time_start", "modify_time_end",
                "payment_time_start", "payment_time_end",
                "refunt_time_start", "refunt_time_end",
            ],
            split_list=["user_id_list", "id_list", "enroll_id_list", "thread_id_list"],
            alias_dict={
                "payment_time_start": "payment_time__gte", "payment_time_end": "payment_time__lte",
                "modify_time_start": "modify_time__gte", "modify_time_end": "modify_time__lte",
                "create_time_start": "create_time__gte", "create_time_end": "create_time__lte",
                "enroll_category_value": "category__value",
                "thread_id_list": "thread_id__in", "enroll_id_list": "enroll_id__in", "id_list": "id__in",
            },
            is_remove_empty=True
        )
        payment_obj = payment_obj.filter(**params).order_by("-id").values(*filter_fields_list)

        # if not need_pagination:
        #     return list(payment_obj), None
        # 分页展示
        paginator = Paginator(payment_obj, size)
        try:
            payment_obj = paginator.page(page)
        except EmptyPage:
            payment_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'

        data = list(payment_obj.object_list)
        enroll_id_list = [item.get("enroll_id", None) for item in data]
        try:
            enroll = db.table('enroll_enroll as e') \
                .select(
                'e.id',
                't.title'
            ) \
                .left_join('thread as t', 't.id', '=', 'e.thread_id') \
                .where_in("e.id", enroll_id_list) \
                .get('u.id')
            if enroll:
                data = JoinList(data, enroll.items, "enroll_id", "id").join()

            data = PaymentService.format_dates(data, ['create_time', "order_time", "modify_time", "payment_time",
                                                      "refunt_time",
                                                      "close_time"])
            user_id_list = [item.get("user_id", None) for item in data]
            user_list, err = DetailInfoService.get_list_detail(user_id_list=user_id_list)
            if user_list:
                data = JoinList(data, user_list, "user_id", "user_id").join()
        except Exception as e:
            write_to_log(
                prefix="支付列表",
                content="支付列表:" + str(data),
                err_obj=str(e)
            )

        return {'total': paginator.count, "page": page, "size": size, 'list': data}, None

    # 退款总接口
    @staticmethod
    def refund(params):
        data = params
        data['transaction_id'] = params['transaction_id']  # 支付单号
        data['refund_fee'] = float(params['refund_amount']) * 100  # 元转分
        data['out_trade_no'] = timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(
            map(str, random.sample(range(0, 9), 4)))
        # 支付方式检查
        if params['payment_method'] == "WECHAT":  # 微信退款
            payment = PaymentWechatService.payment_refund(data)
        else:
            payment = "退款方式不存在"

        return payment

    # 支付总接口
    @staticmethod
    def pay(params):
        data = params
        # 验证支付方式
        payment_method = params.get("payment_method", None)
        # 大写转小写
        payment_method = payment_method.lower()
        payment_method_list = ["applets", "appletsv3", "balance", "wechat_web", 'wechat_app', 'alipay_app']
        if not find(payment_method_list, payment_method):
            return None, "支付方式不存在"

        # 随机生成订单号
        out_trade_no = timezone.now().strftime('%Y%m%d%H%M%S') + ''.join(
            map(str, random.sample(range(0, 9), 4)))
        params['out_trade_no'] = out_trade_no

        # 判断是否是报名订单
        if params.get('enroll_id', ''):
            enroll_data, err_txt = EnrollServices.enroll_detail(params['enroll_id'])
            if err_txt:
                return None, "报名记录不存在"
            data['enroll_id'] = enroll_data['id']
            data['total_fee'] = Decimal(enroll_data['unpaid_amount']) * Decimal('100')  # 元转分

        # 返回datetime格式的时间
        tz = pytz.timezone('Asia/Shanghai')
        now_time = timezone.now().astimezone(tz=tz).strftime("%Y-%m-%d %H:%M:%S")
        now = datetime.strptime(now_time, '%Y-%m-%d %H:%M:%S')

        payment_status_set = PaymentStatus.objects.filter(payment_status="未支付").first()
        payment_data = {
            "transact_no": out_trade_no,
            "order_no": out_trade_no,
            "enroll_id": data['enroll_id'],
            "user_id": data['user_id'],
            "total_amount": int(data['total_fee']),
            "create_time": now,
            "payment_status_id": payment_status_set.id,
            "snapshot": data.get("snapshot", {}),
        }
        if params.get("platform_id", 0):
            platform_set, platform_set_err = UserPlatformService.payment_get_platform_info(
                platform_id=params['platform_id'])
        elif params.get("platform_code", ""):
            platform_set, platform_set_err = UserPlatformService.payment_get_platform_info(
                platform_code=params['platform_code'])

        if platform_set_err:
            data['platform'] = merchant_name
        else:
            data['platform'] = platform_set['platform_name']

        data['currency'] = 'CNY'

        # 金额验证
        if params.get("total_amount", 0):
            data['total_fee'] = Decimal(params.get("total_amount", "0")) * Decimal('100')  # 元转分
        if data['total_fee'] < 1:
            return None, "支付金额不能小于一分钱"
        try:
            # 微信小程序支付
            if payment_method == "applets" or payment_method == "appletsv3":
                payment, err = PaymentWechatService.wechat_jsapi(data, "applets")
            # 公众号支付
            elif payment_method == "wechat_web":
                payment, err = PaymentWechatService.wechat_jsapi(data, "wechat_web")
            elif payment_method == "wechat_app":
                payment, err = PaymentWechatService.wechat_jsapi(data, "wechat_app")
            elif payment_method == "alipay_app":
                payment, err = PaymentAlipayService.app_apy(data)
            # 支付记录写入
            PaymentPayment.objects.create(**payment_data)
        except Exception as e:
            return None, str(e)

        if err:
            return None, err
        return payment, None

    @staticmethod
    def ask_order_status(params):

        try:

            enroll_id = params.get("enroll_id", None)
            if not enroll_id:
                return None, "enroll_id不能为空"
            payment_set = PaymentPayment.objects.filter(**{"enroll_id": enroll_id}).order_by('-id')
            payment_set = payment_set.annotate(payment_status_name=F("payment_status__payment_status"),
                                               payment_code=F("payment_status__payment_code")).first()
            if not payment_set:
                return None, "查询不到该数据"
            payment_obj = model_to_dict(payment_set)
            PaymentStatus_set = PaymentStatus.objects.filter(**{"id": payment_obj['payment_status']}).first()
            PaymentStatus_obj = model_to_dict(PaymentStatus_set)
            return {
                "status": PaymentStatus_obj['payment_code'],
                "transact_no": payment_obj['transact_no'],
                "order_no": payment_obj['order_no']
            }, None
        except Exception as e:
            write_to_log(
                prefix="支付询单接口",
                content="报名id:" + str(enroll_id),
                err_obj=str(e)
            )
            return None, str(e)

    @staticmethod
    def format_dates(items, date_fields):
        for item in items:
            for field in date_fields:
                if field in item and item[field]:
                    try:
                        # 如果字段已经是 datetime 对象，就无需解析
                        if isinstance(item[field], datetime):
                            date = item[field]
                        else:
                            # 尝试解析并格式化日期
                            date = parse(item[field])
                        # 使用 strftime 格式化日期
                        item[field] = date.astimezone(tzlocal()).strftime('%Y-%m-%d %H:%M:%S')
                    except ValueError:
                        # 如果解析失败，保留原来的值
                        pass
        return items
